@@ -6,7 +6,7 @@ module Data.IP (
   -- ** IP data
     IP (..)
   , IPv4, toIPv4, fromIPv4, fromHostAddress, toHostAddress
-  , IPv6, toIPv6, fromIPv6, fromHostAddress6, toHostAddress6
+  , IPv6, toIPv6, toIPv6b, fromIPv6, fromIPv6b, fromHostAddress6, toHostAddress6
   -- ** IP range data
   , IPRange (..)
   , AddrRange (addr, mask, mlen)
